﻿// Interfaz
var ui = {
	/**
   *
   * @param {*} value Valor de la celda
   * @param {*} row Fila
   * @param {*} index indice
   * @returns El código html para representar un circuito
   */
  pagadaFormato: function (value, row, index) {
    if (isEmptyOrSpaces(value)) return;
    valueLow = value.toLowerCase();

    switch (valueLow) {
      case "pagada":
        color = "green";
        icono = "fa fa-check";
        break;
      case "pendiente":
        color = "red";
        icono = "fa fa-ban";
        break;
      default:
        color = "black";
        icono = "remove";
        break;
    }

    html = '<span style="color: ' + color + '"><i class="' + icono + '" aria-hidden="true"></i></span>';

    return html;
  },
    /**
     * Establece el nombre del cliente
     */
    setNombreCliente: function setNombreCliente() {
        var span_nombreCliente = $("#nombre-cliente");
        var nombreCliente = span_nombreCliente.text();
        var nombreClienteLon = nombreCliente.length;

        if (nombreClienteLon >= 15) {
            var nLon = 8;
            nombreCliente = nombreCliente.slice(0, nLon) + "..." + nombreCliente.slice(nombreClienteLon - nLon, nombreClienteLon);
            span_nombreCliente.text(nombreCliente);
        }
    },
    /**
     *
     * @param {Array} arrayJson
     * @param {String} carpeta Si está definina los adjuntos se ponen dentro de una carpeta
     * @param {String} nombreFichero Nombre del fichero para descargar (documentos.zip)
     */
    descargarAdjuntos: function (arrayJson, carpeta, nombreFichero) {
        arrayJson = arrayJson.map(function (adjunto) {
            if (adjunto.oficial === true) return adjunto;
        });

        if (arrayJson.length === 0) {
            alert("No existe ningún documento oficial");
            return;
        }

        if (isEmptyOrSpaces(nombreFichero)) nombreFichero = "documentos.zip";

        var Promise = window.Promise;
        if (!Promise) {
            Promise = JSZip.external.Promise;
        }

        if (JSZip.support.blob) {
            var zip = new JSZip();
            $.each(arrayJson, function (index, adjunto) {
                if (adjunto.oficial === true) {
                    if (isEmptyOrSpaces(carpeta) == false) {
                        var zip = zip.folder(carpeta);
                    }
                    zip.file(adjunto.fichero, urlToPromise(adjunto.url), { binary: true });
                }
            });
            zip.generateAsync({ type: "blob" }, function updateCallback(metadata) {
                console.log(metadata.percent.toFixed(2));
            }).then(
                function callback(blob) {
                    // see FileSaver.js
                    saveAs(blob, nombreFichero);
                },
                function (e) {
                    alert(e);
                }
            );
        }
    },
};

// Tabla de datos
var tabla_datos = {
    /**
     *
     * @param {*} value Valor de la celda
     * @param {*} row Fila
     * @param {*} index indice
     * @returns El código html para representar un circuito
     */
    bs_via: function bs_via(value, row, index) {
        if (isEmptyOrSpaces(value)) return;
        valueLow = value.toLowerCase();
        icono = "";

        switch (valueLow) {
            case "m":
                icono = "fa fa-ship";
                break;
            case "t":
                icono = "fa fa-truck";
                break;
            case "naranja":
                icono = "fa fa-plane";
                break;
            default:
                icono = "";
                break;
        }

        html = ['<span style="color: #3557A5">', '<i class="' + icono + '" aria-hidden="true"></i>', "</span>"].join("");

        return html;
    },
    /**
     *
     * @param {Number} a Número
     * @param {Number} b Número
     * @returns Devuelve la resta para la ordenación
     */
    bs_ordenNumero: function bs_ordenNumero(a, b) {
		if (typeof a != "number") {
		  a = stripNonNumber(a);
		}
		if (typeof b != "number") {
		  b = stripNonNumber(b);
		}

		return a - b;
    },

    /**
     *
     * @param {String} a Cadena 1
     * @param {String} b Cadena 2
     * @returns Devuelve las cadenas en orden
     */
    bs_orderAlfaNum: function bs_orderAlfaNum(a, b) {
        function chunkify(t) {
            const tz = [];
            let y = -1;
            let n = 0;

            for (let i = 0; i <= t.length; i++) {
                const char = t.charAt(i);
                const charCode = char.charCodeAt(0);
                const m = charCode === 46 || (charCode >= 48 && charCode <= 57);
                if (m !== n) {
                    tz[++y] = "";
                    n = m;
                }
                tz[y] += char;
            }

            return tz;
        }

        function stringfy(v) {
            if (typeof v === "number") {
                v = `${v}`;
            }
            if (!v) {
                v = "";
            }
            return v;
        }

        const aa = chunkify(stringfy(a));
        const bb = chunkify(stringfy(b));

        for (let x = 0; aa[x] && bb[x]; x++) {
            if (aa[x] !== bb[x]) {
                const c = Number(aa[x]);
                const d = Number(bb[x]);

                if (c === aa[x] && d === bb[x]) {
                    return c - d;
                }
                return aa[x] > bb[x] ? 1 : -1;
            }
        }
        return aa.length - bb.length;
    },
    /**
     *
     * @param {String} fecha
     * @returns {Date} Fecha
     */
    fechaToDate: function fechaToDate(fecha) {
        if (fecha === "") return new Date();
        fecha = fecha.split("-");
        fecha = fecha.map((valor) => parseInt(valor));
        fecha = new Date(2000 + fecha[2], fecha[1] - 1, fecha[0]);
        return fecha;
    },

    /**
     *
     * @param {String} fecha1 Fecha inicial
     * @param {String} fecha2 Fecha final
     * @returns Valor para ordenar la columna
     */
    bs_fechasOrden: function bs_fechasOrden(fecha1, fecha2) {
        var fechaA = fechaToDate(fecha1);
        var fechaB = fechaToDate(fecha2);

        return fechaA - fechaB;
    },
};

/**
 * @returns Devuelve un array con las filas seleccionas
 */
function getIdSelections() {
    return $.map($table.bootstrapTable("getSelections"), function (row) {
        return row;
    });
}

/*
* Genera la barra de botones de la tablas.
*/
function bs_botones() {
    return {
        btnDescargar: {
            text: "Descargar",
            icon: "fa fa-download",
            event: function () {
                var filas_seleccionadas = getIdSelections();

                if (filas_seleccionadas == 0) return;
                $(".progress-bar")
                    .css("width", 0 + "%")
                    .attr("aria-valuenow", 0);
                $("#barra-progreso").show();

                var documentosArray = [];
                $.each(filas_seleccionadas, function (index, elemento) {
                    if (elemento.documentos.length) {
                        $.each(elemento.documentos, function (index, documento) {
                            documento.codigo = elemento.codigo;
                            documentosArray.push(documento);
                        });
                    }
                });

                var Promise = window.Promise;
                if (!Promise) {
                    Promise = JSZip.external.Promise;
                }

                if (JSZip.support.blob) {
                    var zip = new JSZip();
                    $.each(documentosArray, function (index, adjunto) {						
						zip.folder(adjunto.codigo).file(adjunto.fichero, urlToPromise(adjunto.url), { binary: true });
                    });
                    zip.generateAsync({ type: "blob" }, function updateCallback(metadata) {
                        console.log(metadata.percent.toFixed(2));
                        $(".progress-bar")
                            .css("width", metadata.percent.toFixed(2) + "%")
                            .attr("aria-valuenow", metadata.percent.toFixed(2));
                    }).then(
                        function callback(blob) {
                            // see FileSaver.js
                            saveAs(blob, "documentos.zip");
                            $("#barra-progreso").hide();
                        },
                        function (e) {
                            alert(e);
                        }
                    );
                }
                $btnDescargar.prop("disabled", true);
            },
            attributes: {
                title: "Descargar adjuntos",
                id: "btn-descargar",
                "aria-label": "Descargar",
                disabled: "",
            },
        },
    };
}

/**
 *
 * @param {*} value Valor de la celda
 * @param {*} row Fila
 * @param {*} index indice
 * @returns El código html para representar un circuito
 */
function bs_circuito(value, row, index) {
    if (isEmptyOrSpaces(value)) return;
    valueLow = value.toLowerCase();
    color = "black";

    switch (valueLow) {
        case "verde":
            color = "#008000";
            break;
        case "rojo":
            color = "#d32f2f";
            break;
        case "naranja":
            color = "#f57c00";
            break;
        case "amarillo":
            color = "#fbc02d";
            break;
        default:
            color = "black";
            break;
    }

    html = ['<span style="color: ' + color + '" class="' + color + '">', " " + value + "", "</span>"].join("");

    return html;
}

/**
 *
 * @param {String} fecha
 * @returns {Date} Fecha
 */
function fechaToDate(fecha) {
    if (fecha === "") return new Date();
    fecha = fecha.split("-");
    fecha = fecha.map((valor) => parseInt(valor));
    fecha = new Date(2000 + fecha[2], fecha[1] - 1, fecha[0]);
    return fecha;
}

/**
 *
 * @param {String} fecha1 Fecha inicial
 * @param {String} fecha2 Fecha final
 * @returns Valor para ordenar la columna
 */
function bs_fechasOrden(fecha1, fecha2) {
    var fechaA = fechaToDate(fecha1);
    var fechaB = fechaToDate(fecha2);

    return fechaA - fechaB;
}

/**
 *
 * @param {String} str
 * @returns Verdadero si la cadena está vacía o solo tiene espacios
 */
function isEmptyOrSpaces(str) {
    return str === undefined || str === null || str.match(/^ *$/) !== null;
}

/**
 * Fetch the content and return the associated promise.
 * @param {String} url the url of the content to fetch.
 * @return {Promise} the promise containing the data.
 */
function urlToPromise(url) {
    return new Promise(function (resolve, reject) {
        JSZipUtils.getBinaryContent(url, function (err, data) {
            if (err) {
                reject(err);
            } else {
                resolve(data);
            }
        });
    });
}
